@@ -21,16 +21,22 @@
  * 02110-1301 USA
  */
 
+#include "common.h"
+#include "oauth2plugin.h"
+#include "oauth2tokendata.h"
+
 #include <QUrl>
 #include <QNetworkRequest>
 #include <QNetworkReply>
 #include <QDateTime>
 
-#include <qjson/parser.h>
+#define USE_LIBQJSON (QT_VERSION < QT_VERSION_CHECK(5, 0, 0))
 
-#include "common.h"
-#include "oauth2plugin.h"
-#include "oauth2tokendata.h"
+#if USE_LIBQJSON
+#include <qjson/parser.h>
+#else
+#include <QJsonDocument>
+#endif
 
 using namespace SignOn;
 using namespace OAuth2PluginNS;
@@ -224,8 +230,6 @@ void OAuth2Plugin::process(const SignOn::SessionData &inData,
 {
     Q_D(OAuth2Plugin);
 
-    OAuth2PluginData input;
-
     if ((!mechanism.isEmpty()) && (!mechanisms().contains(mechanism))) {
         emit error(Error(Error::MechanismNotAvailable));
         return;
@@ -269,7 +273,7 @@ void OAuth2Plugin::process(const SignOn::SessionData &inData,
     d->m_password = inData.Secret();
 
     if (mechanism == WEB_SERVER || mechanism == USER_AGENT) {
-        d->m_oauth2Data = inData.data<OAuth2PluginData>();
+        d->m_oauth2Data = data;
         if (mechanism == WEB_SERVER &&
             storedData.contains(REFRESH_TOKEN) &&
             !storedData[REFRESH_TOKEN].toString().isEmpty()) {
@@ -435,8 +439,9 @@ void OAuth2Plugin::serverReply(QNetworkReply *reply)
             }
         }
         // Added to test with facebook Graph API's (handling text/plain content type)
-        else if (reply->rawHeader(CONTENT_TYPE).startsWith(CONTENT_TEXT_PLAIN)){
-            TRACE()<< "text/plain content received";
+        else if (reply->rawHeader(CONTENT_TYPE).startsWith(CONTENT_TEXT_PLAIN) ||
+                 reply->rawHeader(CONTENT_TYPE).startsWith(CONTENT_APP_URLENCODED)) {
+            TRACE()<< reply->rawHeader(CONTENT_TYPE) << "content received";
             QMap<QString,QString> map = parseTextReply(replyContent);
             QByteArray accessToken = map["access_token"].toAscii();
             QByteArray expiresIn = map["expires"].toAscii();
@@ -569,7 +574,26 @@ void OAuth2Plugin::storeResponse(const OAuth2PluginTokenData &response)
     OAuth2TokenData tokens;
     QVariantMap token;
     token.insert(TOKEN, response.AccessToken());
-    token.insert(REFRESH_TOKEN, response.RefreshToken());
+    /* Do not overwrite the refresh token with an empty one: when using the
+     * refresh token to obtain a new access token, the replie could not contain
+     * a refresh token (or contain an empty one).
+     * In such cases, we should re-store the old refresh token.
+     */
+    QString refreshToken;
+    if (response.RefreshToken().isEmpty()) {
+        QVariant tokenVar = d->m_tokens.value(d->m_key);
+        QVariantMap storedData;
+        if (tokenVar.canConvert<QVariantMap>()) {
+            storedData = tokenVar.value<QVariantMap>();
+            if (storedData.contains(REFRESH_TOKEN) &&
+                !storedData[REFRESH_TOKEN].toString().isEmpty()) {
+                refreshToken = storedData[REFRESH_TOKEN].toString();
+            }
+        }
+    } else {
+        refreshToken = response.RefreshToken();
+    }
+    token.insert(REFRESH_TOKEN, refreshToken);
     token.insert(EXPIRY, response.ExpiresIn());
     token.insert(TIMESTAMP, QDateTime::currentDateTime().toTime_t());
     d->m_tokens.insert(d->m_key, QVariant::fromValue(token));
@@ -581,9 +605,15 @@ void OAuth2Plugin::storeResponse(const OAuth2PluginTokenData &response)
 const QVariantMap OAuth2Plugin::parseJSONReply(const QByteArray &reply)
 {
     TRACE();
+    bool ok = false;
+#if USE_LIBQJSON
     QJson::Parser parser;
-    bool ok;
     QVariant tree = parser.parse(reply, &ok);
+#else
+    QJsonDocument doc = QJsonDocument::fromJson(reply);
+    ok = !doc.isEmpty();
+    QVariant tree = doc.toVariant();
+#endif
     if (ok) {
         return tree.toMap();
     }
